import { SelectionRange } from "@codemirror/state";
import { Decoration, ViewPlugin, highlightActiveLine } from "@codemirror/view";
import { FakeSelection } from "./fake-selection";
// @ts-expect-error create is not exposed
const create = highlightActiveLine().create;
const lineDeco = Decoration.line({ attributes: { class: "cm-activeLine fake" } });
export function fakeActiveLine() {
    return ViewPlugin.define(function (view) {
        const instance = create(view);
        // update
        instance.update = function (update) {
            const effects = update.transactions
                .map(tr => tr.effects.filter(e => e.is(FakeSelection)))
                .reduce((a, b) => a.concat(b), []);
            if (effects.length > 0) {
                this.range = SelectionRange.fromJSON(effects[effects.length - 1].value);
            }
            if (update.docChanged || effects.length > 0)
                this.decorations = this.getDeco(update.view);
        };
        // getDeco
        instance.getDeco = function (view) {
            let lastLineStart = -1;
            const deco = [];
            const ranges = view.state.selection.ranges;
            const length = view.state.doc.length;
            // the length check is necessary for rewinding
            if (this.range && this.range.from <= length && this.range.to <= length) {
                // @ts-expect-error ranges is readonly
                view.state.selection.ranges = [this.range];
            }
            else {
                // @ts-expect-error ranges is readonly
                view.state.selection.ranges = [];
            }
            for (const r of view.state.selection.ranges) {
                if (!r.empty)
                    return Decoration.none;
                const line = view.visualLineAt(r.head);
                if (line.from > lastLineStart) {
                    deco.push(lineDeco.range(line.from));
                    lastLineStart = line.from;
                }
            }
            // @ts-expect-error ranges is readonly
            view.state.selection.ranges = ranges;
            return Decoration.set(deco);
        };
        return instance;
    }, {
        decorations: (v) => v.decorations
    });
}
