import { SelectionRange, StateEffect } from "@codemirror/state";
import { ViewPlugin, drawSelection } from "@codemirror/view";
export const FakeSelection = StateEffect.define();
const create = drawSelection()[1].create;
export function fakeSelection() {
    return ViewPlugin.define(function (view) {
        const instance = create(view);
        instance.cursorLayer.classList.add("fake");
        Object.assign(instance.cursorLayer.style, {
            animationIterationCount: "infinite",
            animationName: "cm-blink",
            animationTimingFunction: "steps(1)"
        });
        // intercept read method
        // this is so evil
        const readPos = instance.measureReq.read;
        instance.measureReq.read = function () {
            const ranges = this.view.state.selection.ranges;
            const length = this.view.state.doc.length;
            // the length check is necessary for rewinding
            if (this.range && this.range.from <= length && this.range.to <= length) {
                // @ts-expect-error ranges is readonly
                this.view.state.selection.ranges = [this.range];
            }
            const measure = readPos();
            // @ts-expect-error ranges is readonly
            this.view.state.selection.ranges = ranges;
            for (const c of measure.cursors) {
                const draw = c.draw.bind(c);
                c.draw = function () {
                    const elt = draw();
                    elt.style.display = "block";
                    return elt;
                };
            }
            return measure;
        }.bind(instance);
        // update method
        instance.update = function (update) {
            const effects = update.transactions
                .map(tr => tr.effects.filter(e => e.is(FakeSelection)))
                .reduce((a, b) => a.concat(b), []);
            if (effects.length > 0) {
                this.range = SelectionRange.fromJSON(effects[effects.length - 1].value);
                this.view.requestMeasure(this.measureReq);
            }
        };
        return instance;
    });
}
